;============================================================================;
;                                execute.asm                                 ;

;============================================================================;
;execute                                                                     ;
;    Input     : void                                                        ;
;    Output    : void                                                        ;
;    Destroyed : ALL                                                         ;


execute\open_folder_in_window_1
         bra        switch\right

execute\open_folder_in_window_2
         bra        switch\left

execute
         tst.b      window
         beq.s      execute\open_folder_in_window_1
         cmp.b      #3,window
         beq.s      execute\open_folder_in_window_2

         bsr        save_all_screen
         bsr        wait_for_no_key
         bsr        system\interrupts_on

	 pea        execute\end(pc)	; to return there after a RTS

         bsr        get_handle\folder\current
         move.w     d0,d3
         bsr        get_handle\current
         move.w     d3,d0

         btst.w     #14,10(a2)
         beq.s      execute\not_local
         cmp.w      #3,8(a2)
         bne.s      execute\typed_local
         rts

execute\not_local
execute\typed_local
         jsr        filelib::gettype
         bsr        get_handle\current

         clr.w      tios::kb_vars+$1C

         tst.w      d2
         beq        execute\asm
         cmp.w      #1,d2
         beq.s      execute\library
         cmp.w      #2,d2
         beq.s      execute\program
         cmp.w      #7,d2
         beq.s      execute\viewtext
         cmp.w      #11,d2
         beq.s      execute\viewpic

execute\viewexpr
         jmp        viewlib::showEXPR

execute\viewpic
         lea        string\pic_viewer(pc),a0
         moveq.b    #2,d1
         bra        open_with_viewer

execute\viewtext
         lea        string\text_viewer(pc),a0
         clr.b      d1
         bra        open_with_viewer

execute\program
         bsr        get_handle\current
         move.l     a2,-(a7)
         bsr        get_handle\folder\current
         move.l     a2,-(a7)
         pea        string\pathname(pc)
         pea        string
         jsr        tios::sprintf
         pea        string
         jsr        graphlib::clr_scr
         jsr        userlib::runprog
         lea        24(a7),a7
 	 bsr        system\interrupts_off
         bsr        uninit_mode
         bsr        unalloc_file_list
         bsr        unalloc_folder_list
         bsr        init\program
         bra        wait_for_no_key

execute\library
         lea        error\libs_are_not_exec(pc),a0
         bra.s      execute\asm\error

execute\asm
         move.w     d0,-(a7)
         jsr        userlib::exec
	 addq.l     #2,a7
         tst.w      d2
         beq.s      execute\restore_screen
         cmp.w      #6,d2
         bcc.s      execute\restore_screen
         lea        error_type(pc),a0
         subq.b     #1,d2
	 and.w	    #$FF,d2
	 lsl.w	    #5,d2
	 add.w      d2,a0

execute\asm\error
         move.b     #1,error_flag
         rts

execute\restore_screen
         addq.l     #4,a7
 	 bsr        system\interrupts_off
         bsr        uninit_mode
         bsr        unalloc_file_list
         bsr        unalloc_folder_list
         bsr        init\program
         bra        wait_for_no_key

execute\end
 	 bsr        system\interrupts_off
	 bsr        restore_all_screen
	 bra        wait_for_no_key

;============================================================================;
;open_with_viewer                                                            ;
;    Input     : d0 = file handle                                            ;
;                d1 = default viewer number                                  ;
;                a0 = viewer name                                            ;
;    Output    : void                                                        ;
;    Destroyed : ALL                                                         ;

  ;============================;
  ; default viewers :          ;
  ;                            ;
  ; d1 = 0 : none              ;
  ; d1 = 1 : viewlib::showTEXT ;
  ; d1 = 2 : viewlib::showPIC  ;
  ;============================;

open_with_viewer
         move.b     d1,d5
         clr.w      d1
         move.w     d0,d3               ; handle of file to view

         tst.b      (a0)
         beq.s      open_with_viewer\default

;         jsr        filelib::search
         bsr        search

         tst.w      d0
         beq        open_with_viewer\not_found

         move.w     d0,d6
         DEREF      d0,a2
         cmp.l      #$200000,a2
         bcs.s      open_with_viewer\not_archived
;         move.w     12(a0),d4           ; handle
;         btst.w     #9,10(a0)           ; test if archived
;         beq.s      open_with_viewer\not_archived
;;;
open_with_viewer\archived
         move.w     (a2),d0             ; size
         addq.w     #5,d0               ; + 2 for adjusting + 3 for rounding
         clr.l      d4
         move.w     d0,d4

         move.l     d4,-(a7)
         jsr        tios::HeapAlloc
         addq.l     #4,a7

         tst.w      d0
         beq        errors\not_enough_memory

         move.w     d0,d7
         move.w     d0,d6
         DEREF      d0,a1
         lsr.w      #2,d4               ; rounds to next longword boundary
         subq.w     #1,d4
copy_loop
         move.l     (a2)+,(a1)+
         dbra.s     d4,copy_loop
         DEREF      d7,a2               ; address of viewer
;;;

open_with_viewer\not_archived
         move.w     14(a2),d0
         cmp.w      #$6002,2(a2,d0.w)
         bne.s      open_with_viewer\exit
         move.w     d3,4(a2,d0.w)
open_with_viewer\exit
         move.w     d6,-(a7)
         jsr        userlib::exec
	 addq.l     #2,a7
	 rts

open_with_viewer\default

         move.w     d3,d0
         tst.b      d5
         beq.s      open_with_viewer\none
         cmp.b      #1,d5
         beq.s      open_with_viewer\text_viewer

open_with_viewer\pic_viewer
	 jmp        viewlib::showPIC

open_with_viewer\text_viewer

open_with_viewer\not_found
         lea        error\no_viewer(pc),a0
         move.b     #1,error_flag
open_with_viewer\none
         rts

;                                execute.asm                                 ;
;============================================================================;
