;============================================================================;
;                                  menu.asm                                  ;

;============================================================================;
;menu\open                                                                   ;
;    Input     : d6.w = F1 to F5 key value                                   ;
;    Output    : graphics                                                    ;
;    Destroyed : d0,d1,d2,d3,d4,a0                                              ;

menu\open
         move.w     d6,d0
         sub.w      #268,d0
         cmp.b      menu\number(pc),d0
         beq        menu\move\go_to_top
         move.b     d0,d6
         cmp.b      #$FF,menu\number
         beq.s      menu\open\new
         bsr        menu\clear
         bra.s      menu\open\show
menu\open\new
         move.b     indicators_value(pc),d0
         move.b     d0,indicators_saved_value
         clr.b      indicators_value
         bsr        refresh_indicators
menu\open\show
         move.b     d6,menu\number
         bsr        save_all_screen
         bra        menu\show

;============================================================================;
;menu\invert_title                                                           ;
;    Input     : menu\number.b                                               ;
;    Output    : graphics                                                    ;
;    Destroyed : d0,d1,d2,d3,d4                                              ;

menu\invert_title
         clr.w      d0
         move.b     menu\number(pc),d0
         mulu       #menu_title\width,d0
         addq.b     #1,d0
         moveq.w    #1,d1
         moveq.w    #menu_title\width-2,d2
         cmp.b      #4,menu\number
         bne.s      menu\invert_title\title_size
         subq.b     #1,d2               ; because 'help' is one pixel shorter
menu\invert_title\title_size
         moveq.w    #6,d3
         clr.w      d4
         jmp        graphlib::fill          ; invert the menu title


;============================================================================;
;menu\show                                                                   ;
;    Input     : menu\number.b                                               ;
;    Output    : draws a menu                                                ;
;    Destroyed : d0,d1,d2,d3,d4,a5                                           ;


menu\show
         bsr        menu\invert_title
menu\show_2
         bsr        menu\frame
menu\show_3
         bsr        menu\show_text
         bra        menu\invert_item

menu\show_text
         cmp.b      #$FF,menu\number
         beq        menu\show\exit
         bsr        menu\set_parameters
         move.b     (a5),d0                 ; menu X
         move.b     1(a5),d1                ; menu Y
         move.b     2(a5),d2                ; menu width
         move.b     3(a5),d3                ; menu height
	 movem.w    d0-d2,-(a7)
         addq.b     #1,d0
         addq.b     #1,d1
         subq.b     #2,d2
         subq.b     #2,d3
	 moveq.w    #1,d4	            ; in white
	 jsr        graphlib::fill
         clr.w      -(a7)
         jsr        tios::FontSetSys
         move.w     #4,-(a7)                ; color
         subq.l     #8,a7                   ; get enough space on the stack
         subq.w     #4,14(a7)
         move.w     14(a7),2(a7)            ; Y
         move.b     4(a5),d4                ; number of items
         lsl.b      #1,d4                   ; *2 (item + shortcut)
         addq.b     #1,d4                   ; because it's zeroed
         move.w     12(a7),d0
         add.w      d0,16(a7)
         subq.w     #2,16(a7)
         lea        5(a5),a5                ; a5 points the first item
menu\show_text\find_string
         tst.b      (a5)+
         bne.s      menu\show_text\find_string
         move.l     a5,4(a7)
         move.b     d4,d0
         and.b      #1,d0
         beq.s      menu\show_text\set_shortcut
menu\show_text\set_item
         addq.w     #7,2(a7)
         move.w     12(a7),(a7)
         addq.w     #2,(a7)
         bra.s      menu\show_text\draw
menu\show_text\set_shortcut
         move.w     16(a7),(a7)
         move.l     a5,a0
         move.w     d4,-(a7)
         jsr        graphlib::getlength
         move.w     (a7)+,d4
         sub.w      d3,(a7)
menu\show_text\draw
         jsr        tios::DrawStrXY
         dbf.s      d4,menu\show_text\find_string
         move.w     #1,-(a7)
         jsr        tios::FontSetSys
         lea        20(a7),a7
menu\show\exit
         rts

menu\frame
         cmp.b      #$FF,menu\number
         beq.s      menu\frame\exit
         bsr        menu\set_parameters
         move.b     (a5),d0                 ; menu X
         move.b     1(a5),d1                ; menu Y
         move.b     2(a5),d4                ; menu width
         move.b     3(a5),d5                ; menu height
	 jsr        graphlib::frame
menu\frame\exit
         rts

;============================================================================;
;menu\clear                                                                  ;
;    Input     : menu\number.b                                               ;
;    Output    : void                                                        ;
;    Destroyed : d0,d1,d2,d3,d4,a5                                           ;

menu\clear
         cmp.b      #$FF,menu\number
         beq.s      menu\clear\exit
         bsr        restore_all_screen
         move.b     #$FF,menu\number
         clr.b      menu\item
menu\clear\exit
         rts


;============================================================================;
;menu\switch\right                                                           ;
;    Input     : void                                                        ;
;    Output    : switch to next menu                                         ;
;    Destroyed : d0,d1,d2,d3,d4,a5                                           ;

menu\switch\right
         move.b     menu\number,-(a7)
         bsr        menu\clear
         move.b     (a7)+,menu\number
         cmp.b      #4,menu\number
         bne.s      menu\switch\not_last
         move.b     #-1,menu\number
menu\switch\not_last
         addq.b     #1,menu\number
         bsr        save_all_screen
         bra        menu\show


;============================================================================;
;menu\switch\left                                                            ;
;    Input     : void                                                        ;
;    Output    : switch to next menu                                         ;
;    Destroyed : d0,d1,d2,d3,d4,a5                                           ;

menu\switch\left
         move.b     menu\number,-(a7)
         bsr        menu\clear
         move.b     (a7)+,menu\number
         tst.b      menu\number
         bne.s      menu\switch\not_first
         move.b     #5,menu\number
menu\switch\not_first
         subq.b     #1,menu\number
         bsr        save_all_screen
         bra        menu\show


;============================================================================;
;menu\set_parameters                                                         ;
;    Input     : menu\number.b                                               ;
;    Output    : a5                                                          ;
;    Destroyed : d0                                                          ;

menu\set_parameters
         clr.l      d0
         clr.w      d1
         clr.w      d2
         clr.w      d3
         clr.w      d4
         clr.w      d5
         lea        menu\list(pc),a5
         move.b     menu\number(pc),d0
         lsl.l      #2,d0
         move.l     0(a5,d0.l),a5       ; a5 points the menu
         rts

;============================================================================;
;menu\call_sub_routine                                                       ;
;    Input     : menu\number.b, menu\item.b                                  ;
;    Output    : executes the sub routine                                    ;
;    Destroyed : d6,d7,a0                                                    ;

menu\call_sub_routine
         clr.l      d6
         clr.l      d7
         move.b     menu\number(pc),d6
         move.b     menu\item(pc),d7
menu\call_sub_routine_2
         move.b     indicators_saved_value(pc),d0
         move.b     d0,indicators_value
         move.b     #1,indicators_clear_flag
         bsr        menu\clear
         bsr        draw_info
         lsl.w      #2,d6
         lsl.w      #2,d7
         lea        menu\number_list(pc),a0
         movea.l    0(a0,d6.l),a0
         movea.l    0(a0,d7.l),a0
         jmp        (a0)

;============================================================================;
;menu\1_to_9_shortcut                                                        ;
;    Input     : d6 = [1] to [9] key value                                   ;
;    Output    : executes the sub routine                                    ;
;    Destroyed : d6,d7,a0                                                    ;

menu\1_to_9_shortcut
         sub.b      #49,d6
         bsr        menu\set_parameters
         cmp.b      4(a5),d6
         bhi.s      menu\1_to_9_shortcut\out_of_range
         bsr        menu\invert_item
         move.b     d6,menu\item
         bsr        menu\invert_item
         bsr        wait_for_no_key
         bra.s      menu\call_sub_routine
menu\1_to_9_shortcut\out_of_range
         rts

;============================================================================;
;menu\invert_item                                                            ;
;    Input     : menu\item.b                                                 ;
;    Output    : graphics                                                    ;
;    Destroyed : d0,d1,d2,d3,d4,a5                                           ;


menu\invert_item
         bsr        menu\set_parameters
         move.b     (a5),d0             ; menu X
         move.b     1(a5),d1            ; menu Y
         addq.b     #2,d0
         addq.b     #2,d1
         clr.w      d4
         move.b     menu\item(pc),d4
         mulu       #7,d4
         add.b      d4,d1
         move.b     2(a5),d2
         subq.b     #4,d2
         moveq.w    #6,d3
         clr.b      d4
         jmp        graphlib::fill

;============================================================================;
;menu\move                                                                   ;
;    Input     : d6.w = key value (up or down)                               ;
;    Output    : graphics                                                    ;
;    Destroyed : d0-d5/a5                                                    ;

menu\move\up
         bsr        menu\set_parameters
         tst.b      menu\item
         beq.s      menu\move\go_to_bottom
         bsr        menu\invert_item
         subq.b     #1,menu\item
         bra.s      menu\invert_item

menu\move\down
         bsr        menu\set_parameters
         move.b     4(a5),d0
         cmp.b      menu\item,d0
         beq.s      menu\move\go_to_top
         bsr        menu\invert_item
         addq.b     #1,menu\item
         bra.s      menu\invert_item

menu\move\go_to_top
         bsr        menu\set_parameters
         bsr        menu\invert_item
         clr.b      menu\item
         bra.s      menu\invert_item

menu\move\go_to_bottom
         bsr        menu\set_parameters
         bsr        menu\invert_item
         move.b     4(a5),menu\item
         bra.s      menu\invert_item


;                                  menu.asm                                  ;
;============================================================================;